@@ -17,6 +17,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.             *
  ***************************************************************************/
+#include <QSet>
 #include "multifilecache.h"
 #include <errno.h>
 #include <qdir.h>
@@ -29,6 +30,9 @@
 #include <util/functions.h>
 #include <util/error.h>
 #include <util/log.h>
+#ifdef Q_WS_WIN
+#include <util/win32.h>
+#endif
 #include <torrent/torrent.h>
 #include "cache.h"
 #include "chunk.h"
@@ -36,7 +40,8 @@
 #include "dndfile.h"
 #include "preallocationthread.h"
 #include "movedatafilesjob.h"
-#include <kdebug.h>
+#include "deletedatafilesjob.h"
+#include "piecedata.h"
 
 
 namespace bt
@@ -53,7 +58,7 @@ namespace bt
 		ctmp += bt::DirSeparator();
 #endif
 		
-		for (Uint32 i = 0;i < sl.count() - 1;i++)
+		for (int i = 0;i < sl.count() - 1;i++)
 		{
 			ctmp += sl[i];
 			if (!bt::Exists(ctmp))
@@ -73,6 +78,7 @@ namespace bt
 		else
 			output_dir = this->datadir;
 		files.setAutoDelete(true);
+		dnd_files.setAutoDelete(true);
 	}
 
 
@@ -84,18 +90,13 @@ namespace bt
 		QString file_map = tmpdir + "file_map";
 		if (!bt::Exists(file_map))
 		{
-			QFile fptr(file_map);
-			if (!fptr.open(QIODevice::WriteOnly))
-				throw Error(i18n("Failed to create %1 : %2",file_map,fptr.errorString()));
-			
-			QTextStream out(&fptr);
-			// file map doesn't exist, so create it based upon the output_dir
+			// file map doesn't exist, so just set the path on disk if it has not happened yet
 			Uint32 num = tor.getNumFiles();
 			for (Uint32 i = 0;i < num;i++)
 			{
 				TorrentFile & tf = tor.getFile(i);
-				tf.setPathOnDisk(output_dir + tf.getPath());
-				out << tf.getPathOnDisk() << ::endl;
+				if (tf.getPathOnDisk().isEmpty())
+					tf.setPathOnDisk(output_dir + tf.getUserModifiedPath());
 			}
 			saveFileMap();
 		}
@@ -112,6 +113,16 @@ namespace bt
 				tor.getFile(idx).setPathOnDisk(path);
 				idx++;
 			}
+			
+			// now the user modified paths must come
+			idx = 0;
+			while (!fptr.atEnd() && idx < tor.getNumFiles())
+			{
+				QString path = QString::fromLocal8Bit(fptr.readLine().trimmed()); 
+				if (!path.isEmpty())
+					tor.getFile(idx).setUserModifiedPath(path);
+				idx++;
+			}
 		}
 	}
 	
@@ -130,6 +141,13 @@ namespace bt
 			TorrentFile & tf = tor.getFile(i);
 			out << tf.getPathOnDisk() << ::endl;
 		}
+		
+		// After the actual paths on disk, save the user modified path names
+		for (Uint32 i = 0;i < num;i++)
+		{
+			TorrentFile & tf = tor.getFile(i);
+			out << tf.getUserModifiedPath() << ::endl;
+		}
 	}
 
 	
@@ -140,6 +158,7 @@ namespace bt
 
 	void MultiFileCache::close()
 	{
+		clearPieceCache();
 		files.clear();
 	}
 	
@@ -168,7 +187,15 @@ namespace bt
 					if (dnd_files.contains(i))
 						dnd_files.erase(i);
 					
-					dfd = new DNDFile(dnd_dir + tf.getPath() + ".dnd");
+					QString dnd_path = QString("file%1.dnd").arg(tf.getIndex());
+					QString dnd_file = dnd_dir + dnd_path;
+					if (bt::Exists(dnd_dir + tf.getUserModifiedPath() + ".dnd"))
+					{
+						// old style dnd dir, move the file so that we can keep working 
+						// with the old file
+						bt::Move(dnd_dir + tf.getUserModifiedPath() + ".dnd",dnd_file,true,true);
+					}
+					dfd = new DNDFile(dnd_file,&tf,tor.getChunkSize());
 					dfd->checkIntegrity();
 					dnd_files.insert(i,dfd);
 				}
@@ -198,7 +225,10 @@ namespace bt
 			{
 				DNDFile* dfd = dnd_files.find(i);
 				if (dfd)
-					dfd->changePath(dnd_dir + tf.getPath() + ".dnd");
+				{
+					QString dnd_path = QString("file%1.dnd").arg(tf.getIndex());
+					dfd->changePath(dnd_dir + dnd_path);
+				}
 			}
 		}
 	}
@@ -215,7 +245,7 @@ namespace bt
 		for (Uint32 i = 0;i < num;i++)
 		{
 			TorrentFile & tf = tor.getFile(i);
-			tf.setPathOnDisk(output_dir + tf.getPath());
+			tf.setPathOnDisk(output_dir + tf.getUserModifiedPath());
 			CacheFile* cf = files.find(tf.getIndex());
 			if (cf)
 				cf->changePath(tf.getPathOnDisk());
@@ -223,7 +253,7 @@ namespace bt
 		saveFileMap();
 	}
 	
-	KJob* MultiFileCache::moveDataFiles(const QString & ndir)
+	Job* MultiFileCache::moveDataFiles(const QString & ndir)
 	{
 		if ( !bt::Exists ( ndir ) )
 			bt::MakeDir ( ndir );
@@ -245,11 +275,11 @@ namespace bt
 
 				// check if every directory along the path exists, and if it doesn't
 				// create it
-			MakeFilePath(nd + tf.getPath());
-				
-			if (tf.getPathOnDisk() != nd + tf.getPath())
+			MakeFilePath(nd + tf.getUserModifiedPath());
+			
+			if (QFileInfo(tf.getPathOnDisk()).canonicalPath() != QFileInfo(nd + tf.getUserModifiedPath()).canonicalPath())
 			{
-				job->addMove(tf.getPathOnDisk(),nd + tf.getPath());
+				job->addMove(tf.getPathOnDisk(),nd + tf.getUserModifiedPath());
 				nmoves++;
 			}
 		}
@@ -261,12 +291,11 @@ namespace bt
 		}
 		else
 		{
-			job->startMoving();
 			return job;
 		}
 	}
 	
-	void MultiFileCache::moveDataFilesFinished(KJob* job)
+	void MultiFileCache::moveDataFilesFinished(Job* job)
 	{
 		if (job->error())
 			return;
@@ -274,62 +303,25 @@ namespace bt
 		for ( Uint32 i = 0;i < tor.getNumFiles();i++ )
 		{
 			TorrentFile & tf = tor.getFile(i);
-			tf.setPathOnDisk(new_output_dir + tf.getPath());
+			tf.setPathOnDisk(new_output_dir + tf.getUserModifiedPath());
 			CacheFile* cf = files.find(tf.getIndex());
 			if (cf)
 				cf->changePath(tf.getPathOnDisk());
 			// check for empty directories and delete them
-			DeleteEmptyDirs(output_dir,tf.getPath());
+			DeleteEmptyDirs(output_dir,tf.getUserModifiedPath());
 		}
 	}
 	
-	KJob* MultiFileCache::moveDataFiles(const QMap<TorrentFileInterface*,QString> & files)
+	Job* MultiFileCache::moveDataFiles(const QMap<TorrentFileInterface*,QString> & files)
 	{
 		if (files.count() == 0)
 			return 0;
 		
-		MoveDataFilesJob* job = new MoveDataFilesJob();
-		int nmoves = 0;
-		QMap<TorrentFileInterface*,QString>::const_iterator i = files.begin();
-		while (i != files.end())
-		{
-			TorrentFileInterface* tf = i.key();
-			QString dest = i.value();
-			if (QFileInfo(dest).isDir())
-			{
-				QString path = tf->getPath();
-				if (!dest.endsWith(bt::DirSeparator()))
-					dest += bt::DirSeparator();
-			
-				int last = path.lastIndexOf(bt::DirSeparator());
-				QString dst = dest + path.mid(last+1);
-				if (tf->getPathOnDisk() != dst)
-				{
-					job->addMove(tf->getPathOnDisk(),dst);
-					nmoves++;
-				}
-			}
-			else if (tf->getPathOnDisk() != i.value())
-			{
-				job->addMove(tf->getPathOnDisk(),i.value());
-				nmoves++;
-			}
-			i++;
-		}
-		
-		if (nmoves)
-		{
-			job->startMoving();
-			return job;
-		}
-		else
-		{
-			delete job;
-			return 0;
-		}
+		MoveDataFilesJob* job = new MoveDataFilesJob(files);
+		return job;
 	}
 		
-	void MultiFileCache::moveDataFilesFinished(const QMap<TorrentFileInterface*,QString> & fmap,KJob* job)
+	void MultiFileCache::moveDataFilesFinished(const QMap<TorrentFileInterface*,QString> & fmap,Job* job)
 	{
 		if (job->error())
 			return;
@@ -342,7 +334,7 @@ namespace bt
 			QString dest = i.value();
 			if (QFileInfo(dest).isDir())
 			{
-				QString path = tf->getPath();
+				QString path = tf->getUserModifiedPath();
 				if (!dest.endsWith(bt::DirSeparator()))
 					dest += bt::DirSeparator();
 			
@@ -367,12 +359,31 @@ namespace bt
 		if (!bt::Exists(tmpdir + "dnd"))
 			bt::MakeDir(tmpdir + "dnd");
 
-		// update symlinks
+		QSet<QString> shortened_names;
 		for (Uint32 i = 0;i < tor.getNumFiles();i++)
 		{			
 			TorrentFile & tf = tor.getFile(i);
+#ifndef Q_WS_WIN			
+			// check if the filename is to long
+			if (FileNameToLong(tf.getPathOnDisk()))
+			{
+				QString s = ShortenFileName(tf.getPathOnDisk());
+				Out(SYS_DIO|LOG_DEBUG) << "Path to long " << tf.getPathOnDisk() << endl;
+				// make sure there are no dupes
+				int cnt = 1;
+				while (shortened_names.contains(s))
+				{
+					s = ShortenFileName(tf.getPathOnDisk(),cnt++);
+				}
+				Out(SYS_DIO|LOG_DEBUG) << "Shortened to " << s << endl;
+				
+				tf.setPathOnDisk(s);
+				shortened_names.insert(s);
+			}
+#endif			
 			touch(tf);
 		}
+
 		saveFileMap();
 	}
 	
@@ -380,12 +391,10 @@ namespace bt
 
 	void MultiFileCache::touch(TorrentFile & tf)
 	{
-		QString fpath = tf.getPath();
+		QString fpath = tf.getUserModifiedPath();
 		bool dnd = tf.doNotDownload();
 		// first split fpath by / separator 
 		QStringList sl = fpath.split(bt::DirSeparator());
-		QString dtmp = tmpdir + "dnd" + bt::DirSeparator();
-		MakeFilePath(dtmp + fpath); // make DND dir
 		
 		// then make the file
 		if (!dnd)
@@ -402,38 +411,119 @@ namespace bt
 			}
 		}
 	}
-
-	void MultiFileCache::load(Chunk* c)
+	
+	PieceData* MultiFileCache::createPiece(Chunk* c,Uint32 off,Uint32 length,bool read_only)
 	{
+		PieceData* piece = 0;
 		QList<Uint32> tflist;
 		tor.calcChunkPos(c->getIndex(),tflist);
 		
-		// one file is simple, just mmap it
+		// one file so try to map it
 		if (tflist.count() == 1)
 		{
 			const TorrentFile & f = tor.getFile(tflist.first());
 			CacheFile* fd = files.find(tflist.first());
 			if (!fd)
-				return;
+				return 0;
 			
 			if (Cache::mappedModeAllowed() && mmap_failures < 3)
 			{
-				Uint64 off = FileOffset(c,f,tor.getChunkSize());
-				Uint8* buf = (Uint8*)fd->map(c,off,c->getSize(),CacheFile::READ);
+				Uint64 offset = FileOffset(c,f,tor.getChunkSize()) + off;
+				piece = new PieceData(c,off,length,0,fd);
+				Uint8* buf = (Uint8*)fd->map(piece,offset,length,read_only ? CacheFile::READ : CacheFile::RW);
 				if (buf)
 				{
-					c->setData(buf,Chunk::MMAPPED);
-					// only return when the mapping is OK
-					// if mmap fails we will just load it buffered
-					return;
+					piece->setData(buf);
+					insertPiece(c,piece);
+					return piece;
 				}
 				else
+				{
 					mmap_failures++;
+				}
+
+				delete piece;
+				piece = 0;
 			}
 		}
+			
+		// mmap failed or there are multiple files, so just do buffered
+		Uint8* buf = new Uint8[length];
+		piece = new PieceData(c,off,length,buf,0);
+		insertPiece(c,piece);
+		return piece;
+	}
+	
+	PieceData* MultiFileCache::preparePiece(Chunk* c,Uint32 off,Uint32 length)
+	{
+		PieceData* piece = findPiece(c,off,length);
+		if (piece)
+			return piece;
 		
-		Uint8* data = new Uint8[c->getSize()];
-		Uint64 read = 0; // number of bytes read
+		return createPiece(c,off,length,false);
+	}
+	
+	void MultiFileCache::calculateOffsetAndLength(
+			Uint32 piece_off,Uint32 piece_len,Uint64 file_off,
+			Uint32 chunk_off,Uint32 chunk_len,Uint64 & off,Uint32 & len)
+	{
+		// if the piece offset lies in the range of the current chunk, we need to read data
+		if (piece_off >= chunk_off && piece_off + piece_len <= chunk_off + chunk_len ) 
+		{
+			// The piece lies entirely in the current file
+			off = file_off + (piece_off - chunk_off);
+			len = piece_len;
+		}
+		else if (piece_off >= chunk_off && piece_len < chunk_off + chunk_len)
+		{
+			// The start of the piece lies partially in the current file
+			off = file_off + (piece_off - chunk_off);
+			len = chunk_len - (piece_off - chunk_off);
+		}
+		else if (piece_off < chunk_off && piece_off + piece_len > chunk_off && piece_off + piece_len <= chunk_off + chunk_len)
+		{
+			// The end of the piece lies in the file
+			off = file_off;
+			len = piece_len - (chunk_off - piece_off);
+		}
+		else if (chunk_off >= piece_off && chunk_off + chunk_len < piece_off + piece_len)
+		{
+			// the current file lies entirely in the piece
+			off = file_off;
+			len = chunk_len;
+		}
+	}
+
+	PieceData* MultiFileCache::loadPiece(Chunk* c,Uint32 off,Uint32 length)
+	{
+		PieceData* piece = findPiece(c,off,length);
+		if (piece)
+			return piece;
+
+		// create piece and return it if it is mapped or the create failed
+		piece = createPiece(c,off,length,true);
+		if (!piece || piece->mapped())
+			return piece;
+
+		// Now we need to load it
+		QList<Uint32> tflist;
+		tor.calcChunkPos(c->getIndex(),tflist);
+		
+		// The chunk lies in one file, so it is easy
+		if (tflist.count() == 1)
+		{
+			const TorrentFile & f = tor.getFile(tflist[0]);
+			CacheFile* fd = files.find(tflist[0]);
+			Uint64 piece_off = FileOffset(c,f,tor.getChunkSize()) + off;
+			
+			fd->read(piece->data(),length,piece_off);
+			return piece;
+		}
+
+		// multiple files
+		Uint8* data = piece->data();
+		Uint32 chunk_off = 0; // number of bytes passed of the chunk
+		Uint32 piece_off = 0; // how many bytes read to the piece
 		for (int i = 0;i < tflist.count();i++)
 		{
 			const TorrentFile & f = tor.getFile(tflist[i]);
@@ -443,159 +533,159 @@ namespace bt
 			// first calculate offset into file
 			// only the first file can have an offset
 			// the following files will start at the beginning
-			Uint64 off = 0;
+			Uint64 file_off = 0;
 			if (i == 0)
-				off = FileOffset(c,f,tor.getChunkSize());
+				file_off = FileOffset(c,f,tor.getChunkSize());
 			
-			Uint32 to_read = 0;
-			// then the amount of data we can read from this file
+			Uint32 cdata = 0;
+			// then the amount of data of the chunk which is located in this file 
 			if (tflist.count() == 1)
-				to_read = c->getSize();
+				cdata = c->getSize();
 			else if (i == 0)
-				to_read = f.getLastChunkSize();
+				cdata = f.getLastChunkSize();
 			else if (i == tflist.count() - 1)
-				to_read = c->getSize() - read;
+				cdata = c->getSize() - chunk_off;
 			else
-				to_read = f.getSize();
+				cdata = f.getSize();
 			
+			// if the piece does not lie in this part of the chunk, move on
+			if (off + length <= chunk_off || off >= chunk_off + cdata)
+			{
+				chunk_off += cdata;
+				continue;
+			}
 		
-			// read part of data
+			Uint64 read_offset = 0; // The read offset in the file
+			Uint32 read_length = 0; // how many bytes to read
+			calculateOffsetAndLength(off,length,file_off,chunk_off,cdata,read_offset,read_length);
+
+			Uint8* ptr = data + piece_off; // location to write to
+			piece_off += read_length; 
+			
 			if (fd)
-				fd->read(data + read,to_read,off);
+			{
+				fd->read(ptr,read_length,read_offset);
+			}
 			else if (dfd)
 			{
 				Uint32 ret = 0;
 				if (i == 0)
-					ret = dfd->readLastChunk(data,read,c->getSize());
-				else if (i == tflist.count() - 1)
-					ret = dfd->readFirstChunk(data,read,c->getSize());
+					ret = dfd->readLastChunk(ptr,read_offset - file_off,read_length);
 				else
-					ret = dfd->readFirstChunk(data,read,c->getSize());
+					ret = dfd->readFirstChunk(ptr,read_offset,read_length);
 				
-				if (ret > 0 && ret != to_read)
-					Out(SYS_DIO|LOG_DEBUG) << "Warning : MultiFileCache::load ret != to_read" << endl;
-			}
-			read += to_read;
-		}
-		c->setData(data,Chunk::BUFFERED);
-	}
-
-	
-	bool MultiFileCache::prep(Chunk* c)
-	{
-		// find out in which files a chunk lies
-		QList<Uint32> tflist;
-		tor.calcChunkPos(c->getIndex(),tflist);
-		
-//		Out(SYS_DIO|LOG_DEBUG) << "Prep " << c->getIndex() << endl;
-		if (tflist.count() == 1)
-		{
-			// in one so just mmap it
-			Uint64 off = FileOffset(c,tor.getFile(tflist.first()),tor.getChunkSize());
-			CacheFile* fd = files.find(tflist.first());
-			Uint8* buf = 0;
-			if (fd && Cache::mappedModeAllowed() && mmap_failures < 3)
-			{
-				buf = (Uint8*)fd->map(c,off,c->getSize(),CacheFile::RW);
-				if (!buf)
-					mmap_failures++;
+				if (ret > 0 && ret != read_length)
+					Out(SYS_DIO|LOG_DEBUG) << "Warning : MultiFileCache::loadPiece ret != to_read" << endl;
 			}
 			
-			if (!buf)
-			{
-				// if mmap fails or is not possible use buffered mode
-				c->allocate();
-				c->setStatus(Chunk::BUFFERED);
-			}
-			else
-			{
-				c->setData(buf,Chunk::MMAPPED);
-			}
+			chunk_off += cdata;
 		}
-		else
-		{
-			// just allocate it
-			c->allocate();
-			c->setStatus(Chunk::BUFFERED);
-		}
-		return true;
+
+		return piece;
 	}
 
-	void MultiFileCache::save(Chunk* c)
+	void MultiFileCache::savePiece(PieceData* piece)
 	{
-		QList<Uint32> tflist;
-		tor.calcChunkPos(c->getIndex(),tflist);
-		
-		if (c->getStatus() == Chunk::MMAPPED)
+		// in mapped mode unload the piece if not in use
+		if (piece->mapped())
 		{
-			// mapped chunks are easy
-			CacheFile* fd = files.find(tflist[0]);
-			if (!fd)
+			if (piece->inUse())
 				return;
 			
-			fd->unmap(c->getData(),c->getSize());
-			c->clear();
-			c->setStatus(Chunk::ON_DISK);
+			piece->unload();
+			clearPiece(piece);
+			return; 
+		}
+		
+		Uint8* data = piece->data();
+		if (!data) // this should not happen but just in case
+		{
+			if (!piece->inUse())
+				clearPiece(piece);
 			return;
 		}
-	
-	//	Out(SYS_DIO|LOG_DEBUG) << "Writing to " << tflist.count() << " files " << endl;
-		Uint64 written = 0; // number of bytes written
+		
+		Chunk* c = piece->parentChunk();
+		QList<Uint32> tflist;
+		tor.calcChunkPos(c->getIndex(),tflist);
+		Uint32 chunk_off = 0; // number of bytes passed of the chunk
+		Uint32 piece_off = 0; // how many bytes written from the piece
+		Uint32 off = piece->offset();
+		Uint32 length = piece->length();
+		
 		for (int i = 0;i < tflist.count();i++)
 		{
 			const TorrentFile & f = tor.getFile(tflist[i]);
 			CacheFile* fd = files.find(tflist[i]);
 			DNDFile* dfd = dnd_files.find(tflist[i]);
-
+				
 			// first calculate offset into file
 			// only the first file can have an offset
 			// the following files will start at the beginning
-			Uint64 off = 0;
-			Uint32 to_write = 0;
+			Uint64 file_off = 0;
 			if (i == 0)
-			{
-				off = FileOffset(c,f,tor.getChunkSize());
-			}
-
-			// the amount of data we can write to this file
+				file_off = FileOffset(c,f,tor.getChunkSize());
+			
+			Uint32 cdata = 0;
+			// then the amount of data of the chunk which is located in this file 
 			if (tflist.count() == 1)
-				to_write = c->getSize();
+				cdata = c->getSize();
 			else if (i == 0)
-				to_write = f.getLastChunkSize();
+				cdata = f.getLastChunkSize();
 			else if (i == tflist.count() - 1)
-				to_write = c->getSize() - written;
+				cdata = c->getSize() - chunk_off;
 			else
-				to_write = f.getSize();
+				cdata = f.getSize();
+			
+			// if the piece does not lie in this part of the chunk, move on
+			if (off + length <= chunk_off || off >= chunk_off + cdata)
+			{
+				chunk_off += cdata;
+				continue;
+			}
+		
+			Uint64 write_offset = 0; // The write offset in the file
+			Uint32 write_length = 0; // how many bytes to write
+			calculateOffsetAndLength(off,length,file_off,chunk_off,cdata,write_offset,write_length);
+
+			Uint8* ptr = data + piece_off; // location to read from
+			piece_off += write_length; 
 			
-		//	Out(SYS_DIO|LOG_DEBUG) << "to_write " << to_write << endl;
-			// write the data
 			if (fd)
-				fd->write(c->getData() + written,to_write,off);
+			{
+				fd->write(ptr,write_length,write_offset);
+			}
 			else if (dfd)
 			{
 				if (i == 0)
-					dfd->writeLastChunk(c->getData() + written,to_write);
-				else if (i == tflist.count() - 1)
-					dfd->writeFirstChunk(c->getData() + written,to_write);
+					dfd->writeLastChunk(ptr,write_offset - file_off,write_length);
 				else
-					dfd->writeFirstChunk(c->getData() + written,to_write);
+					dfd->writeFirstChunk(ptr,write_offset,write_length);
 			}
 			
-			written += to_write;
+			chunk_off += cdata;
 		}
 		
-		// set the chunk to on disk and clear it
-		c->clear();
-		c->setStatus(Chunk::ON_DISK);
+		if (!piece->inUse())
+			clearPiece(piece);
 	}
 	
 	void MultiFileCache::downloadStatusChanged(TorrentFile* tf, bool download)
 	{	
 		bool dnd = !download;
 		QString dnd_dir = tmpdir + "dnd" + bt::DirSeparator();
+		QString dnd_path = QString("file%1.dnd").arg(tf->getIndex());
+		QString dnd_file = dnd_dir + dnd_path;
 		// if it is dnd and it is already in the dnd tree do nothing
-		if (dnd && bt::Exists(dnd_dir + tf->getPath() + ".dnd"))
+		if (dnd && bt::Exists(dnd_dir + dnd_path))
 			return;
+		else if (dnd && bt::Exists(dnd_dir + tf->getUserModifiedPath() + ".dnd"))
+		{
+			// old style dnd dir, move the file so that we can keep working 
+			// with the old file
+			bt::Move(dnd_dir + tf->getUserModifiedPath() + ".dnd",dnd_file,true,true);
+			return;
+		}
 		
 		// if it is !dnd and it is already in the output_dir tree do nothing
 		if (!dnd && bt::Exists(tf->getPathOnDisk()))
@@ -606,24 +696,8 @@ namespace bt
 		CacheFile* fd = 0;
 		try
 		{
-			
-			if (dnd && bt::Exists(dnd_dir + tf->getPath()))
+			if (dnd)
 			{
-				// old download, we need to convert it
-				// save first and last chunk of the file
-				saveFirstAndLastChunk(tf,dnd_dir + tf->getPath(),dnd_dir + tf->getPath() + ".dnd");
-				// delete symlink
-				bt::Delete(cache_dir + tf->getPath(),true);
-				bt::Delete(dnd_dir + tf->getPath()); // delete old dnd file
-				
-				files.erase(tf->getIndex());
-				dfd = new DNDFile(dnd_dir + tf->getPath() + ".dnd");
-				dfd->checkIntegrity();
-				dnd_files.insert(tf->getIndex(),dfd);
-			}
-			else if (dnd)
-			{
-				QString dnd_file = dnd_dir + tf->getPath() + ".dnd";
 				// save first and last chunk of the file
 				if (bt::Exists(tf->getPathOnDisk()))
 					saveFirstAndLastChunk(tf,tf->getPathOnDisk(),dnd_file);
@@ -632,15 +706,15 @@ namespace bt
 				bt::Delete(tf->getPathOnDisk(),true);
 				
 				files.erase(tf->getIndex());
-				dfd = new DNDFile(dnd_file);
+				dfd = new DNDFile(dnd_file,tf,tor.getChunkSize());
 				dfd->checkIntegrity();
 				dnd_files.insert(tf->getIndex(),dfd);
 			}
 			else
 			{
 				// recreate the file
-				recreateFile(tf,dnd_dir + tf->getPath() + ".dnd",tf->getPathOnDisk());
-				bt::Delete(dnd_dir + tf->getPath() + ".dnd");
+				recreateFile(tf,dnd_dir + dnd_path,tf->getPathOnDisk());
+				bt::Delete(dnd_dir + dnd_path);
 				dnd_files.erase(tf->getIndex());
 				fd = new CacheFile();
 				fd->open(tf->getPathOnDisk(),tf->getSize());
@@ -659,33 +733,25 @@ namespace bt
 	
 	void MultiFileCache::saveFirstAndLastChunk(TorrentFile* tf,const QString & src_file,const QString & dst_file)
 	{
-		DNDFile out(dst_file);
+		DNDFile out(dst_file,tf,tor.getChunkSize());
 		File fptr;
 		if (!fptr.open(src_file,"rb"))
 			throw Error(i18n("Cannot open file %1 : %2",src_file,fptr.errorString()));
 		
-		Uint32 cs = 0;
-		if (tf->getFirstChunk() == tor.getNumChunks() - 1)
-		{
-			cs = tor.getFileLength() % tor.getChunkSize();
-			if (cs == 0)
-				cs = tor.getChunkSize();
-		}
-		else
-			cs = tor.getChunkSize();
+		Uint32 cs = (tf->getFirstChunk() == tor.getNumChunks() - 1) ? tor.getLastChunkSize() : tor.getChunkSize();
 		
 		Uint8* tmp = new Uint8[tor.getChunkSize()];
 		try
 		{
 			fptr.read(tmp,cs - tf->getFirstChunkOffset());
-			out.writeFirstChunk(tmp,cs - tf->getFirstChunkOffset());
+			out.writeFirstChunk(tmp,0,cs - tf->getFirstChunkOffset());
 			
 			if (tf->getFirstChunk() != tf->getLastChunk())
 			{
 				Uint64 off = FileOffset(tf->getLastChunk(),*tf,tor.getChunkSize());
 				fptr.seek(File::BEGIN,off);
 				fptr.read(tmp,tf->getLastChunkSize());
-				out.writeLastChunk(tmp,tf->getLastChunkSize());
+				out.writeLastChunk(tmp,0,tf->getLastChunkSize());
 			}
 			delete [] tmp;
 		}
@@ -698,7 +764,7 @@ namespace bt
 	
 	void MultiFileCache::recreateFile(TorrentFile* tf,const QString & dnd_file,const QString & output_file)
 	{
-		DNDFile dnd(dnd_file);
+		DNDFile dnd(dnd_file,tf,tor.getChunkSize());
 		
 		// make sure path exists
 		MakeFilePath(output_file);
@@ -709,12 +775,12 @@ namespace bt
 		{
 			bool res = false;
 			
-			#ifdef HAVE_XFS_XFS_H
-				if( (! res) && Cache::useFSSpecificPreallocMethod() )
-				{
-					res = XfsPreallocate(output_file, tf->getSize()) );
-				}
-			#endif
+#ifdef HAVE_XFS_XFS_H
+			if (Cache::preallocateFully())
+			{
+				res = XfsPreallocate(output_file, tf->getSize()) );
+			}
+#endif
 			
 			if(! res)
 			{
@@ -730,15 +796,7 @@ namespace bt
 			}
 		}
 		
-		Uint32 cs = 0;
-		if (tf->getFirstChunk() == tor.getNumChunks() - 1)
-		{
-			cs = tor.getFileLength() % tor.getChunkSize();
-			if (cs == 0)
-				cs = tor.getChunkSize();
-		}
-		else
-			cs = tor.getChunkSize();
+		Uint32 cs = (tf->getFirstChunk() == tor.getNumChunks() - 1) ? tor.getLastChunkSize() : tor.getChunkSize();
 		
 		File fptr;
 		if (!fptr.open(output_file,"r+b"))
@@ -772,7 +830,6 @@ namespace bt
 	
 	void MultiFileCache::preallocateDiskSpace(PreallocationThread* prealloc)
 	{
-		Out(SYS_DIO|LOG_DEBUG) << "MultiFileCache::preallocateDiskSpace" << endl;
 		PtrMap<Uint32,CacheFile>::iterator i = files.begin();
 		while (i != files.end())
 		{
@@ -830,7 +887,7 @@ namespace bt
 			QStringList el = dir.entryList(QDir::AllEntries|QDir::System|QDir::Hidden);
 			el.removeAll(".");
 			el.removeAll("..");
-			if (el.count() == 0)
+			if (el.count() == 0 && dir.exists())
 			{
 				// no childern so delete the directory
 				Out(SYS_GEN|LOG_DEBUG) << "Deleting empty directory : " << path << endl;
@@ -850,15 +907,16 @@ namespace bt
 		QStringList el = dir.entryList(QDir::AllEntries|QDir::System|QDir::Hidden);
 		el.removeAll(".");
 		el.removeAll("..");
-		if (el.count() == 0)
+		if (el.count() == 0 && dir.exists())
 		{
 			Out(SYS_GEN|LOG_DEBUG) << "Deleting empty directory : " << output_dir << endl;
 			bt::Delete(output_dir,true);
 		}
 	}
 	
-	void MultiFileCache::deleteDataFiles()
+	Job* MultiFileCache::deleteDataFiles()
 	{
+		DeleteDataFilesJob* job = new DeleteDataFilesJob(output_dir);
 		for (Uint32 i = 0;i < tor.getNumFiles();i++)
 		{
 			TorrentFile & tf = tor.getFile(i);
@@ -866,12 +924,14 @@ namespace bt
 			if (!tf.doNotDownload())
 			{
 				// first delete the file
-				bt::Delete(fpath);
+				job->addFile(fpath);
 			}
 			
 			// check for subdirectories
-			DeleteEmptyDirs(output_dir,tf.getPath());
+			job->addEmptyDirectoryCheck(tf.getUserModifiedPath());
 		}
+		
+		return job;
 	}
 	
 	Uint64 MultiFileCache::diskUsage()
