@@ -81,6 +81,7 @@ public:
 	unsigned long int idleTime;
 
 	Kopete::StatusMessage statusMessage;
+	KToggleAction* toggleAlwaysVisibleAction;
 };
 
 Contact::Contact( Account *account, const QString &contactId,
@@ -91,29 +92,33 @@ Contact::Contact( Account *account, const QString &contactId,
 
 	d->contactId = contactId;
 	d->metaContact = parent;
+	connect( d->metaContact, SIGNAL(destroyed(QObject*)), this, SLOT(slotMetaContactDestroyed(QObject*)) );
+
 	d->fileCapable = false;
 	d->account = account;
 	d->idleTime = 0;
 	d->icon = icon;
 
+	bool duplicate = false;
 	// If can happend that a MetaContact may be used without a account
 	// (ex: for unit tests or chat window style preview)
 	if ( account )
 	{
-		account->registerContact( this );
+		// Don't register myself contacts because otherwise we can't have own contact in contact list.
+		if ( d->metaContact != Kopete::ContactList::self()->myself() )
+			duplicate = !account->registerContact( this );
+
 		connect( account, SIGNAL( isConnectedChanged() ), SLOT( slotAccountIsConnectedChanged() ) );
 	}
 
 	// Need to check this because myself() may have no parent
 	// Maybe too the metaContact doesn't have a valid protocol()
 	// (ex: for unit tests or chat window style preview)
-	if( parent && protocol() )
-	{
-		connect( parent, SIGNAL( aboutToSave( Kopete::MetaContact * ) ),
-			protocol(), SLOT( slotMetaContactAboutToSave( Kopete::MetaContact * ) ) );
 
+	// if alreadyRegistered is true (which mean that this is duplicate contact) we will not add
+	// parent and the contact will die out on next Kopete restart.
+	if( !duplicate && parent && protocol() )
 		parent->addContact( this );
-	}
 }
 
 Contact::~Contact()
@@ -138,7 +143,7 @@ void Contact::setOnlineStatus( const OnlineStatus &status )
 	if( status == d->onlineStatus )
 		return;
 
-	bool oldCanAcceptFiles = canAcceptFiles();
+	const bool oldCanAcceptFiles = canAcceptFiles();
 	OnlineStatus oldStatus = d->onlineStatus;
 	d->onlineStatus = status;
 
@@ -148,7 +153,8 @@ void Contact::setOnlineStatus( const OnlineStatus &status )
 	if( oldStatus.status() == OnlineStatus::Offline &&
 		status.status() != OnlineStatus::Offline )
 	{
-		setProperty( globalProps->onlineSince(), QDateTime::currentDateTime() );
+		if ( !hasProperty( globalProps->onlineSince().key() ) )
+			setProperty( globalProps->onlineSince(), QDateTime::currentDateTime() );
 		/*kDebug(14010) << "REMOVING lastSeen property for " <<
 			d->displayName << endl;*/
 		removeProperty( globalProps->lastSeen() );
@@ -225,7 +231,7 @@ KMenu* Contact::popupMenu( ChatSession *manager )
 	KMenu *menu = new KMenu();
 
 	QString titleText;
-	QString nick = property( Kopete::Global::Properties::self()->nickName() ).value().toString();
+	const QString nick = property( Kopete::Global::Properties::self()->nickName() ).value().toString();
 	if( nick.isEmpty() )
 		titleText = QString::fromLatin1( "%1 (%2)" ).arg( contactId(), onlineStatus().description() );
 	else
@@ -234,7 +240,7 @@ KMenu* Contact::popupMenu( ChatSession *manager )
 
 	if( metaContact() && metaContact()->isTemporary() && contactId() != account()->myself()->contactId() )
 	{
-		KAction *actionAddContact = new KAction( KIcon("add-user"), i18n( "&Add to Your Contact List" ), menu );
+		KAction *actionAddContact = new KAction( KIcon("list-add-user"), i18n( "&Add to Your Contact List" ), menu );
 		connect( actionAddContact, SIGNAL(triggered(bool)), this, SLOT( slotAddContact() ) );
 
 		menu->addAction(actionAddContact);
@@ -242,8 +248,8 @@ KMenu* Contact::popupMenu( ChatSession *manager )
 	}
 
 	// FIXME: After KDE 3.2 we should make isReachable do the isConnected call so it can be removed here - Martijn
-	bool reach = account()->isConnected() && isReachable();
-	bool myself = (this == account()->myself());
+	const bool reach = account()->isConnected() && isReachable();
+	const bool myself = (this == account()->myself());
 
 	KAction *actionSendMessage = KopeteStdAction::sendMessage( this, SLOT( sendMessage() ), menu );
 	actionSendMessage->setEnabled( reach && !myself );
@@ -277,6 +283,11 @@ KMenu* Contact::popupMenu( ChatSession *manager )
 	{
 		KAction* changeMetaContact = KopeteStdAction::changeMetaContact( this, SLOT( changeMetaContact() ), menu );
 		menu->addAction( changeMetaContact );
+
+		d->toggleAlwaysVisibleAction = new KToggleAction( i18n( "Visible when offline" ), menu );
+		d->toggleAlwaysVisibleAction->setChecked( property( Kopete::Global::Properties::self()->isAlwaysVisible() ).value().toBool() );
+		menu->addAction( d->toggleAlwaysVisibleAction );
+		connect( d->toggleAlwaysVisibleAction, SIGNAL( toggled(bool) ), this, SLOT( toggleAlwaysVisible() ) );
 	}
 
 	menu->addAction( KopeteStdAction::contactInfo( this, SLOT( slotUserInfo() ), menu ) );
@@ -294,6 +305,13 @@ KMenu* Contact::popupMenu( ChatSession *manager )
 	return menu;
 }
 
+void Contact::toggleAlwaysVisible()
+{
+	bool alwaysVisible = property( Kopete::Global::Properties::self()->isAlwaysVisible() ).value().toBool();
+	setProperty( Kopete::Global::Properties::self()->isAlwaysVisible(), !alwaysVisible );
+	d->toggleAlwaysVisibleAction->setChecked( !alwaysVisible );
+}
+
 void Contact::changeMetaContact()
 {
 	KDialog *moveDialog = new KDialog( Kopete::UI::Global::mainWidget() );
@@ -320,6 +338,13 @@ void Contact::changeMetaContact()
 		if(chkCreateNew->isChecked())
 		{
 			mc=new Kopete::MetaContact();
+
+			if ( metaContact() )
+			{	// Add new metaContact to old groups so we don't move it to Top Level group
+				foreach ( Kopete::Group* group, metaContact()->groups() )
+					mc->addToGroup( group );
+			}
+
 			Kopete::ContactList::self()->addMetaContact(mc);
 		}
 		if( mc )
@@ -331,6 +356,14 @@ void Contact::changeMetaContact()
 	moveDialog->deleteLater();
 }
 
+void Contact::slotMetaContactDestroyed( QObject* mc )
+{
+	if (mc != d->metaContact)
+		return;
+
+	d->metaContact = 0;
+}
+
 void Contact::setMetaContact( MetaContact *m )
 {
 	MetaContact *old = d->metaContact;
@@ -340,8 +373,7 @@ void Contact::setMetaContact( MetaContact *m )
 	if( old )
 	{
 		old->removeContact( this );
-		disconnect( old, SIGNAL( aboutToSave( Kopete::MetaContact * ) ),
-			protocol(), SLOT( slotMetaContactAboutToSave( Kopete::MetaContact * ) ) );
+		disconnect( old, SIGNAL(destroyed(QObject*)), this, SLOT(slotMetaContactDestroyed(QObject*)) );
 
 		if(old->contacts().isEmpty())
 		{
@@ -352,22 +384,21 @@ void Contact::setMetaContact( MetaContact *m )
 		{
 			d->metaContact = m; //i am forced to do that now if i want the next line works
 			//remove cached data for this protocol which will not be removed since we disconnected
-			protocol()->slotMetaContactAboutToSave( old );
+			protocol()->serialize( old );
 		}
 	}
 
 	d->metaContact = m;
+	setParent( m );
 
 	if( m )
 	{
 		m->addContact( this );
-		setParent( m );
+		connect( m, SIGNAL(destroyed(QObject*)), this, SLOT(slotMetaContactDestroyed(QObject*)) );
 		// it is necessary to call this write here, because MetaContact::addContact() does not differentiate
 		// between adding completely new contacts (which should be written to kabc) and restoring upon restart
 		// (where no write is needed).
 		KABCPersistence::self()->write( m );
-		connect( d->metaContact, SIGNAL( aboutToSave( Kopete::MetaContact * ) ),
-		protocol(), SLOT( slotMetaContactAboutToSave( Kopete::MetaContact * ) ) );
 	}
 	sync();
 }
@@ -489,7 +520,6 @@ bool Contact::isOnline() const
 	return onlineStatus().isDefinitelyOnline();
 }
 
-
 bool Contact::isFileCapable() const
 {
 	return d->fileCapable;
@@ -536,7 +566,7 @@ QString Contact::toolTip() const
 {
 	Kopete::Property p;
 	QString tip;
-	QStringList shownProps = Kopete::AppearanceSettings::self()->toolTipContents();
+	const QStringList shownProps = Kopete::AppearanceSettings::self()->toolTipContents();
 
 	// --------------------------------------------------------------------------
 	// Fixed part of tooltip
@@ -579,11 +609,11 @@ QString Contact::toolTip() const
 	// Configurable part of tooltip
 
 	// FIXME: It shouldn't use QString to identity the properties. Instead it should use PropertyTmpl::key()
-	for(QStringList::Iterator it=shownProps.begin(); it!=shownProps.end(); ++it)
+	for(QStringList::ConstIterator it=shownProps.constBegin(); it!=shownProps.constEnd(); ++it)
 	{
 		if((*it) == Kopete::Global::Properties::self()->fullName().key())
 		{
-			QString name = formattedName();
+			const QString name = formattedName();
 			if(!name.isEmpty())
 			{
 				tip += i18nc("@label:textbox formatted name",
@@ -592,7 +622,7 @@ QString Contact::toolTip() const
 		}
 		else if ((*it) == Kopete::Global::Properties::self()->idleTime().key())
 		{
-			QString time = formattedIdleTime();
+			const QString time = formattedIdleTime();
 			if(!time.isEmpty())
 			{
 				tip += i18nc("@label:textbox formatted idle time",
@@ -601,7 +631,7 @@ QString Contact::toolTip() const
 		}
 		else if ((*it) == QString::fromLatin1("homePage"))
 		{
-			QString url = property(*it).value().toString();
+			const QString url = property(*it).value().toString();
 			if(!url.isEmpty())
 			{
 				tip += i18nc("@label:textbox formatted url",
@@ -611,7 +641,7 @@ QString Contact::toolTip() const
 		}
 		else if ((*it) == Kopete::Global::Properties::self()->statusTitle().key() )
 		{
-			QString statusTitle = property(*it).value().toString();
+			const QString statusTitle = property(*it).value().toString();
 			if(!statusTitle.isEmpty())
 			{
 				tip += i18nc("@label:textbox formatted status title",
@@ -620,7 +650,7 @@ QString Contact::toolTip() const
 		}
 		else if ((*it) == Kopete::Global::Properties::self()->statusMessage().key() )
 		{
-			QString statusmsg = property(*it).value().toString();
+			const QString statusmsg = property(*it).value().toString();
 			if(!statusmsg.isEmpty())
 			{
 				tip += i18nc("@label:textbox formatted status message",
@@ -756,13 +786,13 @@ void Kopete::Contact::slotUnblock()
 
 void Kopete::Contact::setNickName( const QString &name )
 {
-	QString oldNickName = nickName();
+	const QString oldNickName = nickName();
 	setProperty( Kopete::Global::Properties::self()->nickName(), name );
 }
 
 QString Kopete::Contact::nickName() const
 {
-	QString nick = property( Kopete::Global::Properties::self()->nickName() ).value().toString();
+	const QString nick = property( Kopete::Global::Properties::self()->nickName() ).value().toString();
 	if( !nick.isEmpty() )
 		return nick;
 
