@@ -260,7 +260,7 @@ static void gone_insecure(void *opdata, ConnContext *context){
 	OtrlChatInterface::self()->emitGoneSecure(((Kopete::ChatSession*)opdata), 0);
 	Kopete::ChatSession *session= ((Kopete::ChatSession*)opdata);
 	Kopete::Message msg( session->members().first(), session->account()->myself() );
-	msg.setHtmlBody( i18n("<b>OTR Session ended. The conversation is now insecure!</b>") );
+	msg.setHtmlBody( i18n("<b>OTR Session ended. Note that the conversation is now insecure.</b>") );
 	msg.setDirection( Kopete::Message::Internal );
 	session->appendMessage( msg );
 }
@@ -297,7 +297,7 @@ static int max_message_size(void *opdata, ConnContext *context){
 	
 	kDebug(14318) << session->protocol()->pluginId();
 
-	if( session->protocol()->pluginId() == "MSNProtocol" ){
+	if( session->protocol()->pluginId() == "WlmProtocol" ){
 		return 1409;
 	} else if( session->protocol()->pluginId() == "ICQProtocol" ){
 		return 1274;
@@ -357,14 +357,14 @@ OtrlChatInterface::~ OtrlChatInterface(){
 }
 
 
-KDE_EXPORT OtrlChatInterface *OtrlChatInterface::self(){
+OtrlChatInterface *OtrlChatInterface::self(){
 	if( !mSelf ){
 		new OtrlChatInterface();
 	}
 	return mSelf;
 }
 
-KDE_EXPORT void OtrlChatInterface::setPlugin( Kopete::Plugin *plugin ){
+void OtrlChatInterface::setPlugin( Kopete::Plugin *plugin ){
 	chatPlugin = plugin;
 }
 
@@ -375,7 +375,7 @@ OtrlUserState OtrlChatInterface::getUserstate(){
 }
 
 
-KDE_EXPORT int OtrlChatInterface::decryptMessage( QString *msg, const QString &accountId,
+int OtrlChatInterface::decryptMessage( QString *msg, const QString &accountId,
 	const QString &protocol, const QString &contactId , Kopete::ChatSession *chatSession){
 
 	int ignoremessage;
@@ -413,7 +413,7 @@ KDE_EXPORT int OtrlChatInterface::decryptMessage( QString *msg, const QString &a
 				abortSMP( context, chatSession );
 			} else {
 				kDebug(14318) << "Update SMP state: 1Q";
-				new AuthenticationWizard( chatSession->view()->mainWidget(), context, chatSession, false, QString((char*)tlvs->data) );
+				new AuthenticationWizard( chatSession->view(true)->mainWidget(), context, chatSession, false, QString((char*)tlvs->data) );
 			}
 		}
 
@@ -424,7 +424,7 @@ KDE_EXPORT int OtrlChatInterface::decryptMessage( QString *msg, const QString &a
 				abortSMP( context, chatSession );
 			} else {
 				kDebug(14318) << "Update SMP state: 1 ";
-				new AuthenticationWizard( chatSession->view()->mainWidget(), context, chatSession, false );
+				new AuthenticationWizard( chatSession->view(true)->mainWidget(), context, chatSession, false );
 			}
 		}
 		tlv = otrl_tlv_find(tlvs, OTRL_TLV_SMP2);
@@ -487,7 +487,7 @@ KDE_EXPORT int OtrlChatInterface::decryptMessage( QString *msg, const QString &a
 					AuthenticationWizard::findWizard(chatSession)->finished(true, true);
 					kDebug(14318) << "trust found";
 					Kopete::Message msg( chatSession->members().first(), chatSession->account()->myself() );
-					msg.setHtmlBody( i18n("<b>Authentication successful. The conversation is now secure!</b>") );
+					msg.setHtmlBody( i18n("<b>Authentication successful. The conversation is now secure.</b>") );
 					msg.setDirection( Kopete::Message::Internal );
 					chatSession->appendMessage( msg );
 					OtrlChatInterface::self()->emitGoneSecure( chatSession, 2 );
@@ -495,7 +495,7 @@ KDE_EXPORT int OtrlChatInterface::decryptMessage( QString *msg, const QString &a
 					AuthenticationWizard::findWizard(chatSession)->finished(false, false);
 					kDebug(14318) << "trust _NOT_ found";
 					Kopete::Message msg( chatSession->members().first(), chatSession->account()->myself() );
-					msg.setHtmlBody( i18n("<b>Authentication failed. The conversation is now insecure!</b>") );
+					msg.setHtmlBody( i18n("<b>Authentication failed. Note that the conversation is now insecure.</b>") );
 					msg.setDirection( Kopete::Message::Internal );
 					chatSession->appendMessage( msg );
 					OtrlChatInterface::self()->emitGoneSecure( chatSession, 1 );
@@ -505,10 +505,15 @@ KDE_EXPORT int OtrlChatInterface::decryptMessage( QString *msg, const QString &a
 		}
 		tlv = otrl_tlv_find(tlvs, OTRL_TLV_SMP_ABORT);
 		if (tlv) {
+			kDebug(14318) << "other end aborted SMP";
 			Kopete::Message msg( chatSession->members().first(), chatSession->account()->myself() );
-			msg.setHtmlBody( i18n("<b>Authentication error!</b>") );
+			msg.setHtmlBody( i18n("<b>Authentication error.</b>") );
 			msg.setDirection( Kopete::Message::Internal );
 			chatSession->appendMessage( msg );
+			AuthenticationWizard *currentWizard = AuthenticationWizard::findWizard(chatSession);
+			if(currentWizard){
+				currentWizard->aborted();
+			}
 			context->smstate->nextExpected = OTRL_SMP_EXPECT1;
 		}
 	
@@ -531,18 +536,18 @@ KDE_EXPORT int OtrlChatInterface::decryptMessage( QString *msg, const QString &a
 	return 2; // internal OTR message. Ignore it.
 }
 
-KDE_EXPORT QString *OtrlChatInterface::encryptMessage( QString *msg, const QString &accountId,
+int OtrlChatInterface::encryptMessage( QString *msg, const QString &accountId,
 	const QString &protocol, const QString &contactId , Kopete::ChatSession *chatSession ){
 	int err;
-	char *newMessage;
-	char *fragment;
+	char *newMessage = 0;
+	char *fragment = 0;
 
 	if( otrl_proto_message_type( msg->toLocal8Bit() ) == OTRL_MSGTYPE_NOTOTR ){
-		msg->replace( QString('<'), QString("&lt;") );
+//		msg->replace( QString('<'), QString("&lt;") );
 		err = otrl_message_sending( userstate, &ui_ops, chatSession, accountId.toLocal8Bit(), protocol.toLocal8Bit(), contactId.toLocal8Bit(), msg->toUtf8(), NULL, &newMessage, NULL, NULL );
 	
 		if( err != 0 ){
-			*msg = i18n("Encryption error");
+			return -1;
 		} else if( newMessage ){
 
 			/* Fragment the message if needed */
@@ -563,18 +568,18 @@ KDE_EXPORT QString *OtrlChatInterface::encryptMessage( QString *msg, const QStri
 				otrl_message_free( newMessage );
 				otrl_message_free( fragment );
 			}
+			OtrlMessageType type = otrl_proto_message_type( msg->toLocal8Bit() );
+			if( type == OTRL_MSGTYPE_TAGGEDPLAINTEXT ){
+				return 1; // Message is still plaintext, but tagged for opportunistic mode
+			}
+			return 0; // Encrypted successfully
 		}
 	}
-	OtrlMessageType type = otrl_proto_message_type( msg->toLocal8Bit() );
-	if( type == OTRL_MSGTYPE_NOTOTR | type == OTRL_MSGTYPE_TAGGEDPLAINTEXT ){
-		msg->replace( QString("&lt;"), QString("<") );		
-	}
-
-	
-	return msg;
+    
+	return 2; // Message is still plaintext. Better not touching it
 }
 
-KDE_EXPORT QString OtrlChatInterface::getDefaultQuery( const QString &accountId ){
+QString OtrlChatInterface::getDefaultQuery( const QString &accountId ){
 	char *message;
 	message = otrl_proto_default_query_msg( accountId.toLatin1(), OTRL_POLICY_ALLOW_V2 );
 	QString msg( message );
@@ -582,7 +587,7 @@ KDE_EXPORT QString OtrlChatInterface::getDefaultQuery( const QString &accountId
 	return msg;
 }
 
-KDE_EXPORT void OtrlChatInterface::disconnectSession( Kopete::ChatSession *chatSession ){
+void OtrlChatInterface::disconnectSession( Kopete::ChatSession *chatSession ){
 	otrl_message_disconnect( userstate, &ui_ops, chatSession, chatSession->account()->accountId().toLatin1(), chatSession->account()->protocol()->displayName().toLatin1(), chatSession->members().first()->contactId().toLocal8Bit() );
 	OtrlChatInterface::self()->emitGoneSecure( chatSession, 0 );
 
@@ -594,12 +599,12 @@ KDE_EXPORT void OtrlChatInterface::disconnectSession( Kopete::ChatSession *chatS
 
 }
 
-KDE_EXPORT bool OtrlChatInterface::shouldDiscard( const QString &message ){
+bool OtrlChatInterface::shouldDiscard( const QString &message ){
 	if( !message.isEmpty() && !message.isNull() ){
 //		kDebug(14318) << otrl_proto_message_type( message.toLatin1() );
 		switch( otrl_proto_message_type( message.toLatin1() ) ){
 			case OTRL_MSGTYPE_TAGGEDPLAINTEXT:
-			case OTRL_MSGTYPE_UNKNOWN: // Fragmented messages seem to be type UNKNOWN
+//			case OTRL_MSGTYPE_UNKNOWN: // Fragmented messages seem to be type UNKNOWN
 			case OTRL_MSGTYPE_NOTOTR:
 				return false;
 			default:
@@ -611,12 +616,12 @@ KDE_EXPORT bool OtrlChatInterface::shouldDiscard( const QString &message ){
 }
 
 
-KDE_EXPORT void OtrlChatInterface::setPolicy( OtrlPolicy policy ){
+void OtrlChatInterface::setPolicy( OtrlPolicy policy ){
 	confPolicy = policy;
 }
 
 
-KDE_EXPORT int OtrlChatInterface::privState( Kopete::ChatSession *session ){
+int OtrlChatInterface::privState( Kopete::ChatSession *session ){
 	ConnContext *context;
 	
 	context = otrl_context_find(userstate, session->members().first()->contactId().toLocal8Bit(), session->account()->accountId().toLocal8Bit(), session->account()->protocol()->displayName().toLocal8Bit(), 0, NULL, NULL, NULL);
@@ -637,7 +642,7 @@ KDE_EXPORT int OtrlChatInterface::privState( Kopete::ChatSession *session ){
 	return 0;
 }
 
-KDE_EXPORT QString  OtrlChatInterface::formatContact(const QString &contactId){
+QString  OtrlChatInterface::formatContact(const QString &contactId){
 	
 	Kopete::MetaContact *metaContact = Kopete::ContactList::self()->findMetaContactByContactId(contactId);
 	if( metaContact ){
@@ -649,7 +654,7 @@ KDE_EXPORT QString  OtrlChatInterface::formatContact(const QString &contactId){
 	return contactId;
 }
 
-KDE_EXPORT void OtrlChatInterface::verifyFingerprint( Kopete::ChatSession *session ){
+void OtrlChatInterface::verifyFingerprint( Kopete::ChatSession *session ){
 	ConnContext *context;
 
 	context = otrl_context_find( userstate, session->members().first()->contactId().toLocal8Bit(), session->account()->accountId().toLocal8Bit(), session->protocol()->displayName().toLocal8Bit(), 0, NULL, NULL, NULL);
@@ -694,7 +699,7 @@ bool OtrlChatInterface::isVerified( Kopete::ChatSession *session ){
 	}
 }
 
-KDE_EXPORT void OtrlChatInterface::checkFilePermissions( const QString &file ){
+void OtrlChatInterface::checkFilePermissions( const QString &file ){
 	if( QFile::exists( file ) ){
 		QFile privkeys( file );
 		QFileInfo privkeysInfo( privkeys );
@@ -741,7 +746,7 @@ void OtrlChatInterface::abortSMP( ConnContext *context, Kopete::ChatSession *ses
 		emitGoneSecure( session, 1 );
 
 		Kopete::Message msg( session->members().first(), session->account()->myself() );
-		msg.setHtmlBody( i18n("<b>Authentication aborted. The conversation is now insecure!</b>") );
+		msg.setHtmlBody( i18n("<b>Authentication aborted. Note that the conversation is now insecure.</b>") );
 		msg.setDirection( Kopete::Message::Internal );
 		session->appendMessage( msg );
 	}
