@@ -88,7 +88,7 @@ YahooAccount::YahooAccount(YahooProtocol *parent, const QString& accountId)
 	m_openInboxAction = new KAction( KIcon("mail-folder-inbox"), i18n( "Open Inbo&x..." ), this );
         //, "m_openInboxAction" );
 	QObject::connect(m_openInboxAction, SIGNAL( triggered(bool) ), this, SLOT( slotOpenInbox() ) );
-	m_openYABAction = new KAction( KIcon("x-office-address-book"), i18n( "Open &Addressbook..." ), this );
+	m_openYABAction = new KAction( KIcon("x-office-address-book"), i18n( "Open &Address book..." ), this );
         //, "m_openYABAction" );
 	QObject::connect(m_openYABAction, SIGNAL( triggered(bool) ), this, SLOT( slotOpenYAB() ) );
 	m_editOwnYABEntry = new KAction( KIcon("document-properties"), i18n( "&Edit my contact details..."), this );
@@ -539,7 +539,7 @@ void YahooAccount::connectWithPassword( const QString &passwd )
 		return;
 	}
 
-	QString server = configGroup()->readEntry( "Server", "scs.msg.yahoo.com" );
+	QString server = configGroup()->readEntry( "Server", "scsa.msg.yahoo.com" );
 	int port = configGroup()->readEntry( "Port", 5050 );
 
 	initConnectionSignals( MakeConnections );
@@ -561,7 +561,7 @@ void YahooAccount::disconnect()
 	if ( isConnected() )
 	{
 		kDebug(YAHOO_GEN_DEBUG) <<  "Attempting to disconnect from Yahoo server ";
-		
+
 		disconnected( Manual );
 		m_session->close();
 		static_cast<YahooContact *>( myself() )->setOnlineStatus( m_protocol->Offline );
@@ -570,6 +570,7 @@ void YahooAccount::disconnect()
 		for ( it = contacts().constBegin(); it != itEnd; ++it )
 			static_cast<YahooContact *>( it.value() )->setOnlineStatus( m_protocol->Offline );
 
+		static_cast<YahooContact*>( myself() )->setOnlineStatus( m_protocol->Offline );
 	}
 	else
 	{       //make sure we set everybody else offline explicitly, just for cleanup
@@ -579,6 +580,8 @@ void YahooAccount::disconnect()
 		QHash<QString,Kopete::Contact*>::ConstIterator it, itEnd = contacts().constEnd();
 		for ( it = contacts().constBegin(); it != itEnd; ++it )
 			static_cast<YahooContact*>( it.value() )->setOnlineStatus( m_protocol->Offline );
+
+		static_cast<YahooContact*>( myself() )->setOnlineStatus( m_protocol->Offline );
 	}
 
 	initConnectionSignals( DeleteConnections );
@@ -640,7 +643,7 @@ void YahooAccount::fillActionMenu( KActionMenu *actionMenu )
 
 YahooContact *YahooAccount::contact( const QString &id )
 {
-	return static_cast<YahooContact *>(contacts()[id]);
+	return static_cast<YahooContact *>(contacts().value( id ));
 }
 
 bool YahooAccount::createContact(const QString &contactId, Kopete::MetaContact *parentContact )
@@ -672,7 +675,7 @@ bool YahooAccount::createChatContact(const QString &nick)
 
 void YahooAccount::sendFile( YahooContact *to, const KUrl &url )
 {
-	QFile file( url.path() );
+	QFile file( url.toLocalFile() );
 
 	Kopete::Transfer *transfer = Kopete::TransferManager::transferManager()->addTransfer ( to,
 		url.fileName(), file.size(), to->userId(), Kopete::FileTransferInfo::Outgoing );
@@ -975,7 +978,7 @@ void YahooAccount::slotStealthStatusChanged( const QString &who, Yahoo::StealthS
 		kDebug(YAHOO_GEN_DEBUG) << "contact " << who << " doesn't exist.";
 		return;
 	}
-	kc->setStealthed( state == Yahoo::StealthActive );
+	kc->updateStealthed();
 }
 
 QString YahooAccount::prepareIncomingMessage( const QString &messageText )
@@ -1207,8 +1210,7 @@ void YahooAccount::prepareConference( const QString &who )
 	QHash<QString,Kopete::Contact*>::ConstIterator it, itEnd = contacts().constEnd();
 	for( it = contacts().constBegin(); it != itEnd; ++it )
 	{
-		if( it.value() != myself() )
-			buddies.push_back( it.value()->contactId() );
+		buddies.push_back( it.value()->contactId() );
 	}
 
 	YahooInviteListImpl *dlg = new YahooInviteListImpl( Kopete::UI::Global::mainWidget() );
@@ -1560,31 +1562,23 @@ void YahooAccount::slotGameNotify( const QString & /* who */, int /* stat */ )
 //	kDebug(YAHOO_GEN_DEBUG) ;
 }
 
-void YahooAccount::slotMailNotify( const QString& from, const QString& /* subject */, int cnt )
+void YahooAccount::slotMailNotify( const QString& from, const QString&  subject , int cnt )
 {
-//	kDebug(YAHOO_GEN_DEBUG) << "Mail count: " << cnt;
-
-	if ( cnt > m_currentMailCount && from.isEmpty() )
+	kDebug(YAHOO_GEN_DEBUG) << "Mail count: " << cnt;
+	
+	if ( cnt > 0 && from.isEmpty() )
 	{
-#ifdef __GNUC__
-#warning Fix KNotification here
-#endif
-#if 0
 		QObject::connect(KNotification::event( QLatin1String("yahoo_mail"), i18np( "You have one unread message in your Yahoo inbox.",
 			"You have %1 unread messages in your Yahoo inbox.", cnt ), QPixmap() , 0 ),
 		                 SIGNAL(activated(unsigned int ) ) , this, SLOT( slotOpenInbox() ) );
-#endif
+
 		m_currentMailCount = cnt;
 	}
-	else if ( cnt > m_currentMailCount )
+	else if ( cnt > 0 )
 	{	kDebug(YAHOO_GEN_DEBUG) << "attempting to trigger event";
-#ifdef __GNUC__
-#warning Fix KNotification here
-#endif
-#if 0
-		QObject::connect(KNotification::event( QLatin1String("yahoo_mail"), i18n( "You have a message from %1 in your Yahoo inbox.", from)
+	
+		QObject::connect(KNotification::event( QLatin1String("yahoo_mail"), i18n( "You have a message from %1 in your Yahoo inbox. <br><br>Subject: %2", from, subject )
 		                                       , QPixmap() , 0 ), SIGNAL(activated(unsigned int ) ) , this, SLOT( slotOpenInbox() ) );
-#endif
 		m_currentMailCount = cnt;
 	}
 }
@@ -1699,9 +1693,9 @@ void YahooAccount::slotGotBuddyIconRequest( const QString & who )
 
 void YahooAccount::setBuddyIcon( const KUrl &url )
 {
-	kDebug(YAHOO_GEN_DEBUG) << "Url: " << url.path();
-	QString s = url.path();
-	if ( url.path().isEmpty() )
+	kDebug(YAHOO_GEN_DEBUG) << "Url: " << url.toLocalFile();
+	QString s = url.toLocalFile();
+	if ( url.toLocalFile().isEmpty() )
 	{
 		myself()->removeProperty( Kopete::Global::Properties::self()->photo() );
 		myself()->removeProperty( YahooProtocol::protocol()->iconRemoteUrl );
@@ -1712,7 +1706,7 @@ void YahooAccount::setBuddyIcon( const KUrl &url )
 	}
 	else
 	{
-		QImage image( url.path() );
+		QImage image( url.toLocalFile() );
 		QString newlocation( KStandardDirs::locateLocal( "appdata", "yahoopictures/"+ url.fileName().toLower() ) ) ;
 		QFile iconFile( newlocation );
 		QByteArray data;
